// FlashCard.cpp : implementation file
//
// FlashQuiz v1.0
// Copyright  2000 Ziff Davis Media, Inc.
// Written by Gregory A. Wolking.

#include "stdafx.h"
#include "flashquiz.h"
#include "FlashCard.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CFlashQuizApp theApp;
extern CFlashQuizDoc theDoc;

/////////////////////////////////////////////////////////////////////////////
// CFlashCard dialog


CFlashCard::CFlashCard(CWnd* pParent /*=NULL*/)
	: CDialog(CFlashCard::IDD, pParent)
{
	//{{AFX_DATA_INIT(CFlashCard)
	//}}AFX_DATA_INIT
	m_pQuestion = NULL;
	m_bPreviewMode = FALSE;
	m_intReduceTimer = -1;
	m_bDisableTimer = FALSE;
	m_bHasCustomFont = FALSE;
	m_bIsMaximized = FALSE;
	m_hBitmap = NULL;
}


void CFlashCard::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFlashCard)
	DDX_Control(pDX, IDC_lblEnter, m_lblEnter);
	DDX_Control(pDX, IDC_fraStatus, m_fraStatus);
	DDX_Control(pDX, IDCANCEL, m_cmdCancel);
	DDX_Control(pDX, IDC_txtResponse, m_txtResponse);
	DDX_Control(pDX, IDC_lblScore, m_lblScore);
	DDX_Control(pDX, IDC_lblQuizTime, m_lblQuizTime);
	DDX_Control(pDX, IDC_lblQuestionTime, m_lblQuestionTime);
	DDX_Control(pDX, IDC_lblCumulative, m_lblCumulative);
	DDX_Control(pDX, IDC_txtQuestion, m_txtQuestion);
	DDX_Control(pDX, IDC_lblAnswer, m_lblAnswer);
	DDX_Control(pDX, IDC_fraPicture, m_fraPicture);
	DDX_Control(pDX, IDC_cmdWrong, m_cmdWrong);
	DDX_Control(pDX, IDC_cmdShow, m_cmdShow);
	DDX_Control(pDX, IDC_cmdRight, m_cmdRight);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CFlashCard, CDialog)
	//{{AFX_MSG_MAP(CFlashCard)
	ON_BN_CLICKED(IDC_cmdRight, OncmdRight)
	ON_BN_CLICKED(IDC_cmdShow, OncmdShow)
	ON_BN_CLICKED(IDC_cmdWrong, OncmdWrong)
	ON_WM_SIZE()
	ON_EN_SETFOCUS(IDC_txtQuestion, OnSetfocustxtQuestion)
	ON_EN_KILLFOCUS(IDC_txtQuestion, OnKillfocustxtQuestion)
	//}}AFX_MSG_MAP
	ON_MESSAGE(MY_UPDATE_TIMER, OnUpdateTimer)
	ON_MESSAGE(MY_QUESTION_TIMEOUT, OnQuestionTimeout)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFlashCard message handlers

BOOL CFlashCard::OnInitDialog() 
{
	CString strTemp, strImageFile;

	CDialog::OnInitDialog();
	m_intTextStart = m_intTextEnd = 0;
	m_bAnswerShown = FALSE;
	m_bDisableTimer = FALSE;
	if (m_bPreviewMode)
	{
		m_cmdRight.ShowWindow(SW_HIDE);
		m_cmdWrong.ShowWindow(SW_HIDE);
		m_cmdCancel.SetWindowText("End Preview");
	}
	if (m_pQuestion->m_bCustomFont)
	{
		if (m_fontCustom.CreateFontIndirect(m_pQuestion->m_pFont))
		{
			m_bHasCustomFont = TRUE;
			m_txtQuestion.SetFont(&m_fontCustom);
			m_lblAnswer.SetFont(&m_fontCustom);
		}
	}
	m_intActualScore = m_pQuestion->m_intScore;
	Update_Point_Value();
	if (m_intQuestion == 1 || m_bPreviewMode)
	{
		m_lblCumulative.SetWindowText("Score so far: none.");
	}
	else
	{
		strTemp.Format("Score so far: %u of %u points (%u%%)",
			theDoc.m_intTotalScore,
			theDoc.m_intPossibleScore,
			(100 * theDoc.m_intTotalScore) / theDoc.m_intPossibleScore);
		m_lblCumulative.SetWindowText(strTemp);
	}
	if (theDoc.m_bTimeLimit)
	{
		theApp.Update_Quiz_Time(m_lblQuizTime);
	}
	else
	{
		m_lblQuizTime.ShowWindow(SW_HIDE);
	}
	// Note: Quiz editor will not allow a flashcard with no
	// bitmap _and_ no question text.
	m_bShowGraphic = !m_pQuestion->m_strBitmapFile.IsEmpty();
	m_bShowText = !m_pQuestion->m_strQuestion.IsEmpty();
	if (m_bShowGraphic)
	{
		if (theApp.Find_Image_File(m_pQuestion->m_strBitmapFile, strImageFile))
			m_hBitmap = (HBITMAP) LoadImage(NULL, strImageFile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
		else
		{
			m_bDisableTimer = TRUE;
			strTemp.Format("Error:\nCould not find or open image file %s", "FlashQuiz", strImageFile);
			MessageBox(strTemp, "FlashQuiz", MB_ICONINFORMATION);
			m_bDisableTimer = FALSE;
			m_hBitmap = NULL;
		}
	}
	if (m_bShowText)
		m_txtQuestion.SetWindowText(m_pQuestion->m_strQuestion);
	Set_Question_Position();
	m_intTimeLeft = m_pQuestion->m_intTime;
	if (m_intTimeLeft == 0)
	{
		m_lblQuestionTime.ShowWindow(SW_HIDE);
	}
	else
	{
		Update_Question_Time();
	}
	if (m_pQuestion->m_intTimeDeduct > 0)
	{
		m_intReduceTimer = 0;
	}
	if (m_bIsMaximized)
		ShowWindow(SW_SHOWMAXIMIZED);
	m_pQuestion->m_bPresented = TRUE;
	m_txtResponse.SetFocus();
	return FALSE;
}

// Rearranges controls according to window size.
void CFlashCard::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	// Control sizes, margins, and "gaps" in DLUs.
	CRect	rectShow(0, 0, 126, 14),
			rectMargins(0, 0, 7, 7),
			rectButton(0, 0, 50, 14),
			rectLabel1(0, 0, 150, 8),
			rectLabel2(0, 0, 137, 8),
			rectStatus(0, 0, 323, 29),
			rectGap2(0, 0, 9, 9),
			rectAnswer(0, 0, 323, 34),
			rectEnter(0, 0, 80, 8);
	if (m_cmdRight.m_hWnd == NULL)
		return;

	// Set or clear "maximized" flag accordingly.
	m_bIsMaximized = (nType == SIZE_MAXIMIZED);
	// Convert DLUs to pixels.
	MapDialogRect(rectShow);
	MapDialogRect(rectMargins);
	MapDialogRect(rectButton);
	MapDialogRect(rectStatus);
	MapDialogRect(rectAnswer);
	MapDialogRect(rectLabel1);
	MapDialogRect(rectLabel2);
	MapDialogRect(rectGap2);
	MapDialogRect(rectEnter);

	// "Show me the answer" button is centered horizontally at the
	// bottom edge.
	rectShow.OffsetRect(rectMargins.right - rectShow.left, cy - rectShow.bottom - rectMargins.bottom);
	m_cmdShow.MoveWindow(rectShow);
	// "Right" and "Wrong" buttons are side by side at the bottom left corner.
	rectButton.OffsetRect(rectMargins.right, cy - rectButton.bottom - rectMargins.bottom);
	m_cmdRight.MoveWindow(rectButton);
	rectButton.OffsetRect(rectButton.Width() + rectMargins.right, 0);
	m_cmdWrong.MoveWindow(rectButton);
	// "Abort" button is at the bottom right corner.
	rectButton.OffsetRect(cx - rectButton.right - rectMargins.right, 0);
	m_cmdCancel.MoveWindow(rectButton);
	// "Status" frame goes just above the buttons.
	rectStatus.left = rectAnswer.left = rectMargins.right;
	rectStatus.right = rectAnswer.right = cx - rectMargins.right;
	rectStatus.OffsetRect(0, rectButton.top - rectStatus.bottom - rectMargins.bottom);
	m_fraStatus.MoveWindow(rectStatus);
	// Position the labels within the "Status" frame.
	rectLabel1.OffsetRect(rectStatus.left - rectLabel1.left + rectMargins.right, rectStatus.top - rectLabel1.top + rectGap2.bottom);
	m_lblScore.MoveWindow(rectLabel1);
	rectLabel2.OffsetRect(rectStatus.right - rectLabel2.right - rectMargins.right, rectLabel1.top - rectLabel2.top);
	m_lblQuestionTime.MoveWindow(rectLabel2);
	rectLabel1.OffsetRect(0, rectGap2.bottom);
	m_lblCumulative.MoveWindow(rectLabel1);
	rectLabel2.OffsetRect(0, rectGap2.bottom);
	m_lblQuizTime.MoveWindow(rectLabel2);
	// "Answer" field goes just above the "Status" frame.
	rectAnswer.OffsetRect(0, rectStatus.top - rectAnswer.bottom - rectGap2.bottom);
	m_lblAnswer.MoveWindow(rectAnswer);
	// "Enter response" text box goes just above the "Answer" label.
	rectAnswer.OffsetRect(0, -rectAnswer.Height() - rectGap2.bottom);
	m_txtResponse.MoveWindow(rectAnswer);
	// "Enter response" label goes just above that, at the left margin.
	rectEnter.OffsetRect(rectMargins.right, rectAnswer.top - rectEnter.bottom - rectMargins.bottom);
	rectEnter.right = cx - rectMargins.right;
	m_lblEnter.MoveWindow(rectEnter);
	// Arrange the question text and/or graphic to fill the top of the window.
	Set_Question_Position();
}

//////////////////////////////
// Control event handlers.

void CFlashCard::OncmdRight() 
{
	theDoc.m_intTotalScore += m_intActualScore;
	m_pQuestion->m_intActualScore = m_intActualScore;
	EndDialog(IDOK);
}

void CFlashCard::OncmdShow() 
{
	m_bAnswerShown = TRUE;
	m_lblAnswer.SetWindowText(m_pQuestion->m_strAnswers[0]);
	if (!m_bPreviewMode)
	{
		m_cmdRight.ShowWindow(SW_SHOW);
		m_cmdWrong.ShowWindow(SW_SHOW);
	}
	m_cmdShow.ShowWindow(SW_HIDE);
	m_cmdRight.SetFocus();
	m_cmdRight.SetButtonStyle(BS_DEFPUSHBUTTON);
}

void CFlashCard::OncmdWrong() 
{
	m_pQuestion->m_intActualScore = 0;
	EndDialog(IDOK);
}

void CFlashCard::OnCancel()
{
	if (!m_bPreviewMode)
	{
		if (Query_Abort())
			return;
	}
	else
		EndDialog(IDCANCEL);
}

//////////////////////////////////
// App-specific message handlers.

// Handles "time expired" message.
LONG CFlashCard::OnQuestionTimeout(WPARAM, LPARAM)
{
	theApp.ShowQuestionTimeout(this);
	EndDialog(IDOK);
	return 0L;
}

// Handles message posted by parent window's "master" timer handler.
LONG CFlashCard::OnUpdateTimer(WPARAM, LPARAM)
{
	CString strTemp;

	// If "Abort quiz" message box is active, or user has already hit "Show me the answer":
	if (m_bDisableTimer || m_bAnswerShown)
	{
		if (theDoc.m_bTimeLimit)		// If quiz is timed,
			++theDoc.m_intTimeLeft;		// increment elapsed time.
		return 0L;						// Done.
	}
	if (m_intReduceTimer >= 0)			// Reduce score option enabled?
	{
		++m_intReduceTimer;				// Increment score reduction timer.
		if (m_intReduceTimer >= m_pQuestion->m_intTimeDeduct)	// Limit reached?
		{
			m_intReduceTimer = 0;								// Yes .. reset timer.
			m_intActualScore -= m_pQuestion->m_intScoreDeduct;	// Deduct points from score,
			if (m_intActualScore < 1)							// but minimum score is always 1 point.
				m_intActualScore = 1;
			Update_Point_Value();								// Refresh display.
		}
	}
	if (theDoc.m_bTimeLimit)				// If quiz is timed,
	{
		if (theDoc.m_intTimeLeft < 0)		// Has total time gone negative?
			EndDialog(IDCANCEL);			// Yes -- end the dialog.
		else
			theApp.Update_Quiz_Time(m_lblQuizTime);	// No -- update time display.
	}
	if (m_intTimeLeft > 0)					// Is this question timed?
	{
		--m_intTimeLeft;								// Yes -- decrement time left.
		Update_Question_Time();
		if (m_intTimeLeft == 0)							// Time expired?
		{
			m_bDisableTimer = TRUE;
			PostMessage(MY_QUESTION_TIMEOUT, 0, 0);
		}
	}
	return 0L;
}


////////////////////////////////////
// CDialog member function overrides.

// Perform cleanup before our window is destroyed.
BOOL CFlashCard::DestroyWindow() 
{
	if (m_bHasCustomFont)					// If custom font was assigned,
	{
		m_txtQuestion.SetFont(GetFont(), FALSE);		// Revert to default font.
		m_lblAnswer.SetFont(GetFont(), FALSE);
		m_fontCustom.DeleteObject();		// Delete the font object.
		m_bHasCustomFont = FALSE;			// Clear the flag.
	}
	if (m_hBitmap != NULL)
	{
		DeleteObject(m_hBitmap);
		m_hBitmap = NULL;
	}
	return CDialog::DestroyWindow();
}

////////////////////////////////
// Helper functions.

// Presets quiz abort confirmation prompt.
BOOL CFlashCard::Query_Abort()
{
	m_bDisableTimer = TRUE;		// Suspend timer while prompt is presented.
	if (MessageBox("Are you sure you want to abort this quiz?", "FlashQuiz", MB_YESNO | MB_ICONQUESTION) == IDYES)
	{
		EndDialog(IDCANCEL);	// If user aborts, don't worry about the timer.
		return TRUE;
	}
	return (m_bDisableTimer = FALSE);	// If user doesn't abort, resume the timer.
}

// Updates point value display.
void CFlashCard::Update_Point_Value()
{
	CString strTemp;

	if (m_intActualScore == 1)
	{
		m_lblScore.SetWindowText("This question is worth 1 point.");
	}
	else
	{
		strTemp.Format("This question is worth %u points.", m_intActualScore);
		m_lblScore.SetWindowText(strTemp);
	}
}

// Updates question time remaining display.
void CFlashCard::Update_Question_Time()
{
	CString strTemp;

	strTemp.Format("Time left for this question: %u seconds", m_intTimeLeft);
	m_lblQuestionTime.SetWindowText(strTemp);
}


// Positions the question text and/or graphic controls.
void CFlashCard::Set_Question_Position(void)
{
	CRect rect1, rect2, rectMargins(0, 0, 7, 7);

	GetClientRect(&rect1);
	// Controls fit between the top of the client area and the top of the "Enter answer" label.
	m_lblEnter.GetWindowRect(&rect2);		// Get size of the label.
	ScreenToClient(&rect2);					// Convert to client coordinates.
	if (!m_bShowGraphic)	// No bitmap?
	{
		m_fraPicture.ShowWindow(SW_HIDE);		// Hide the graphic.
		rect2.bottom = rect2.top - 2;			// Center the text.
		rect2.top = rect1.top + rectMargins.bottom;
		rect2.right = (rect2.Width() / 2) + rect2.left;
		rect2.OffsetRect(((rect1.Width() - rect2.Width()) / 2), 0);
		m_txtQuestion.MoveWindow(&rect2);
	}
	else
	{
		if (!m_bShowText)	// No question text?
		{
			m_txtQuestion.ShowWindow(SW_HIDE);		// Hide the text.
			rect2.bottom = rect2.top - 2;		// Center the graphic.
			rect2.top = rect1.top + rectMargins.bottom;
			rect2.right = (rect2.Width() / 2) + rect2.left;
			rect2.OffsetRect((rect1.Width() - rect2.Width()) / 2, 0);
			m_fraPicture.MoveWindow(&rect2);
		}
		else									// Got both bitmap and text
		{
			// Size the graphic and text equally across the top of the dialog.
			rect2.bottom = rect2.top - 2;
			rect2.top = rect1.top + rectMargins.bottom;
			rect2.right = (rect2.Width() / 2) + rect2.left - (rectMargins.right / 2);
			m_fraPicture.MoveWindow(&rect2);
			rect2.OffsetRect(rect1.right - rect2.right - rectMargins.right, 0);
			m_txtQuestion.MoveWindow(&rect2);
		}
		// Since the size of the picture control may have changed,
		// reset (and therefore re-scale) its image.
		m_fraPicture.Set_Scaled_Bitmap(m_hBitmap, FALSE);
		// Control will resize itself to fit the bitmap, so we need to
		// re-center it in the desired area.
		m_fraPicture.GetWindowRect(rect1);
		ScreenToClient(rect1);
		rect1.OffsetRect((rect2.Width() - rect1.Width()) / 2, (rect2.Height() - rect1.Height()) / 2);
		m_fraPicture.MoveWindow(&rect1);
	}
}

void CFlashCard::OnSetfocustxtQuestion() 
{
	m_txtQuestion.SetSel(m_intTextStart, m_intTextEnd);
}

void CFlashCard::OnKillfocustxtQuestion() 
{
	m_txtQuestion.GetSel(m_intTextStart, m_intTextEnd);	
}
