// FlashQuiz.cpp : Defines the class behaviors for the application.
//
// FlashQuiz v1.0
// Copyright  2000 Ziff Davis Media, Inc.
// Written by Gregory A. Wolking.

#include "stdafx.h"
#include "FlashQuiz.h"
#include "FlashQuizDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const char	szEncrypt[] = "frunobulax";
const char	szDocType[] = "FlashQuiz.Document";
char		g_szBuffer[2048];
// Global defaults
int g_intDefaultQuestionTime;
int g_intDefaultTimeDeduct;
int g_intDefaultScoreDeduct;

/////////////////////////////////////////////////////////////////////////////
// CFlashQuizApp

BEGIN_MESSAGE_MAP(CFlashQuizApp, CWinApp)
	//{{AFX_MSG_MAP(CFlashQuizApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFlashQuizApp construction

CFlashQuizApp::CFlashQuizApp()
{
	m_bEditFile = FALSE;
	m_strFileName.Empty();
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CFlashQuizApp object

CFlashQuizApp theApp;

////////////////////////////////////////////////
// The one and only CFlashQuizDoc object

CFlashQuizDoc theDoc;

/////////////////////////////////////////////////////////////////////////////
// CFlashQuizApp initialization

BOOL CFlashQuizApp::InitInstance()
{
	CString strTemp(m_lpCmdLine);	// Get a copy of the command line.
	CString	strTemp2;
	int		iPos;

	Set_Associations();
	// Command line processing
	strTemp.TrimLeft();			// Remove leading & trailing spaces
	strTemp.TrimRight();
	strTemp.MakeLower();		// Convert to lowercase.
	iPos = strTemp.Find("/uninstall");
	if (iPos >= 0)
	{
		iPos = ::MessageBox(NULL, "This will remove FlashQuiz from your system.\n"
			"Are you sure?", "FlashQuiz Uninstall", MB_ICONQUESTION | MB_YESNO);
		if (iPos == IDYES)
			Do_Uninstall();
		return FALSE;
	}
	iPos = strTemp.Find("/edit");	// Look for the "/edit" switch
	if (iPos >= 0)					// If it's there,
	{
		m_bEditFile = TRUE;				// Set flag.
		iPos += 5;						// Move past it.
		strTemp2 = strTemp.Mid(iPos);	// Rest of line should be the filename.
		strTemp2.TrimLeft();
		strTemp2.TrimRight();
		m_strFileName = strTemp2;
	}
	else
		m_strFileName = strTemp;	// Otherwise, only parm should be a filename.
	CFlashQuizDlg dlg;				// Create the main window.
	m_pMainWnd = &dlg;				// Set pointer.
	dlg.DoModal();					// Launch the dialog.
	return FALSE;					// Done when dialog closes.
}


// Scrambles the specified text string.
// Return value: Pointer to (newly allocated) buffer containing the
//               scrambled text, or NULL if function fails.
BYTE * CFlashQuizApp::Scramble(CString& strText)
{
	int		i,
			j,
			iMax,
			iText;
	BYTE	*pData = NULL;

	// Get string lengths.
	iText = strText.GetLength();
	iMax = strlen(szEncrypt) - 1;
	// Allocate array for scrambled text.
	pData = new BYTE[iText];
	if (pData == NULL)				// If allocation failed,
		return pData;				// Return NULL to caller.
	// Copy text to buffer.
	for (i = 0; i < iText; i++)
		pData[i] = strText[i];
	// "Scramble" the buffer context by XORing with each character
	// in the "scrambler" string. Note that this rotates through
	// the "scrambler" string in reverse order.
	j = iMax;
	for (i = 0; i < iText; i++)
	{
		pData[i] = pData[i] ^ szEncrypt[j];
		if ((--j) < 0)
			j = iMax;
	}
	// Return pointer to scrambled text.
	return pData;
}

// "Unscrambles" the specified text using the specified password.
// If successful, result is assigned to the CString passed by reference as strResult.
// Return value: TRUE if success, FALSE if failure.
//               If FALSE, strResult has not been modified.
BOOL CFlashQuizApp::UnScramble(BYTE * pData, int iDataLen, CString& strResult)
{
	int		i,
			j,
			iMax;
	BYTE	*pNewData = NULL;

	// Allocate array to hold unscrambled text.
	pNewData = new BYTE[iDataLen + 1];
	if (pNewData == NULL)
		return FALSE;
	iMax = strlen(szEncrypt) -1;
	// Unscramble the text, using the same algorithm as Scramble().
	j = iMax;
	for (i = 0; i < iDataLen; i++)
	{
		pNewData[i] = pData[i] ^ szEncrypt[j];
		if ((--j) < 0)
			j = iMax;
	}
	// Terminate text with zero byte.
	pNewData[iDataLen] = 0;
	// Assign result.
	strResult = pNewData;
	// Deallocate temporary array.
	delete [] pNewData;
	// Signal success to caller.
	return TRUE;
}

int CFlashQuizApp::ExitInstance() 
{
	return CWinApp::ExitInstance();
}

// Fills the passsed integer vector with a random series
// of unique values ranging from 0 to (iCount - 1).
// Note that the vector pointed to by pIndexArray must be large enough
// to accommodate iCount elements, but there is no need to initialize
// the contents of the vector.
//
// Returns FALSE if unable to allocate a temporary "flag" array.
BOOL CFlashQuizApp::Randomize_List(int *pIndexArray, int iCount)
{
	int		i, iIndex;
	BOOL	*bFlagArray = NULL;

	srand(GetTickCount());			// Seed the rand() library function.
	bFlagArray = new BOOL[iCount];	// Allocate flag array.
	if (bFlagArray == NULL)			// Allocation failed?
	{
		return FALSE;				// Tell caller.
	}
	else								// Otherwise,
	{
		for (i = 0; i < iCount; i++)	// Initialize flag array to FALSE.
			bFlagArray[i] = FALSE;
	}
	// For each element in pIndexArray:
	for (i = 0; i < iCount; i++)
	{
		while (TRUE)					// Loop until element is found.
		{
			iIndex = rand();				// Get a random number.
			if (iIndex < iCount)			// Less than count?
			{
				if (!bFlagArray[iIndex])		// Yes .. value already used?
				{
					pIndexArray[i] = iIndex;	// No -- put value into array.
					bFlagArray[iIndex] = TRUE;	// Mark value as used.
					break;						// Proceed with next array element.
				}
			}
		}
	}
	delete [] bFlagArray;	// Deallocate flag array.
	return TRUE;			// Signal success to caller.
}

// Called by Flash Card and Multiple Choice dialogs.
void CFlashQuizApp::Update_Quiz_Time(CStatic &m_lblDisplay)
{
	int iHours, iMinutes, iSeconds;
	CString strTemp;

	iSeconds = theDoc.m_intTimeLeft;
	iHours = iSeconds / 3600;
	iSeconds -= (iHours * 3600);
	iMinutes = iSeconds / 60;
	iSeconds -= (iMinutes * 60);
	if (iHours > 0)
		strTemp.Format("Time left for this quiz: %02u:%02u:%02u", iHours, iMinutes, iSeconds);
	else
		strTemp.Format("Time left for this quiz: %02u:%02u", iMinutes, iSeconds);
	m_lblDisplay.SetWindowText(strTemp);
}

// Called by Flash Card and Multiple Choice dialogs.
void CFlashQuizApp::ShowQuestionTimeout(CDialog* pParent, LPARAM lParam)
{
	if (pParent != NULL)
	{
		if (pParent->m_hWnd != NULL)
		{
			if (lParam == 0)
				pParent->MessageBox("Time limit for this question has expired.", "FlashQuiz", MB_ICONINFORMATION);
			else
				pParent->MessageBox("Time limit for this quiz has expired.", "FlashQuiz", MB_ICONINFORMATION);
		}
	}
}

// Checks to see if associations are defined and
// sets (or resets) them if appropriate.
void CFlashQuizApp::Set_Associations()
{
	LONG	lResult;
	int		iResult;
	DWORD	dwDisposition;
	HKEY	hKey1, hKey2, hKey3, hKey4;
	CString strTemp1, strTemp2;
	BOOL	bFound = FALSE;

	// Get fully-qualified path to our .EXE file.
	strTemp1 = m_pszHelpFilePath;
	iResult = strTemp1.ReverseFind('.');
	strTemp2 = strTemp1.Left(iResult);
	strTemp2.MakeLower();
	strTemp2 += ".exe";

	// First, make sure our Uninstall command is registered.
	lResult = RegCreateKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\FlashQuiz", 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey1, &dwDisposition);
	if (lResult == 0)
	{
		RegSetValueEx(hKey1, "DisplayName", 0, REG_SZ, (CONST BYTE *) "FlashQuiz", 10);
		strTemp1.Format("\"%s\" /uninstall", strTemp2);
		RegSetValueEx(hKey1, "UninstallString", 0, REG_SZ, (CONST BYTE *) (LPCTSTR) strTemp1, strTemp1.GetLength() + 1);
		RegCloseKey(hKey1);
	}
	// Is there a key for the .FQZ extension?
	bFound = Check_Association();
	// If association exists does not appear to be pointing to us,
	if (!bFound)
	{
		// ask user if he wants to change it.
		iResult = ::MessageBox(NULL, "Do you wish to associate the .FQZ extension with FlashQuiz?", "FlashQuiz", MB_ICONQUESTION | MB_YESNO);
		if (iResult != IDYES) // Done unless user says "Yes".
			return;
	}
	// Make sure that HKCR\.fqz exists.
	lResult = RegCreateKeyEx(HKEY_CLASSES_ROOT, ".fqz", 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey1, &dwDisposition); 
	if (lResult == 0)
	{
		// Make sure HCKR\.fqz points to our document type.
		Set_Default_Value(hKey1, szDocType);
		RegCloseKey(hKey1);
		// Create document type key.
		lResult = RegCreateKeyEx(HKEY_CLASSES_ROOT, szDocType, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey1, &dwDisposition); 
		if (lResult == 0)
		{
			// Set "friendly name" for file type.
			Set_Default_Value(hKey1, "Flash Quiz");
			// Create key for default icon.
			lResult = RegCreateKeyEx(hKey1, "DefaultIcon", 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey2, &dwDisposition);
			if (lResult == 0)
			{
				// Build default icon statement.
				strTemp1 = strTemp2 + ",1";
				// Assign it to the key's default value.
				Set_Default_Value(hKey2, strTemp1);
				// Done with "DefaultIcon" key.
				RegCloseKey(hKey2);
			}
			// Create the "Shell" key.
			lResult = RegCreateKeyEx(hKey1, "Shell", 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey2, &dwDisposition); 
			if (lResult == 0)
			{
				// Create key for the "Open" command.
				Set_Default_Value(hKey2, "Open");
				lResult = RegCreateKeyEx(hKey2, "Open", 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey3, &dwDisposition); 
				if (lResult == 0)
				{
					// Set default text for the menu.
					Set_Default_Value(hKey3, "&Open");
					// Create "command" subkey.
					lResult = RegCreateKeyEx(hKey3, "command", 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey4, &dwDisposition);
					if (lResult == 0)
					{
						// Build command line.
						strTemp1.Format("\"%s\" %%1", strTemp2);
						// Install it.
						Set_Default_Value(hKey4, strTemp1);
						// Done with "command" key.
						RegCloseKey(hKey4);
					}
					// Done with "Open" key.
					RegCloseKey(hKey3);
				}
				// Create key for "Edit" command.
				lResult = RegCreateKeyEx(hKey2, "Edit", 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey3, &dwDisposition); 
				if (lResult == 0)
				{
					// Set its menu text.
					Set_Default_Value(hKey3, "&Edit quiz");
					// Create "comand" subkey.
					lResult = RegCreateKeyEx(hKey3, "command", 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey4, &dwDisposition);
					if (lResult == 0)
					{
						// Build & install "edit" command line.
						strTemp1.Format("\"%s\" /edit %%1", strTemp2);
						Set_Default_Value(hKey4, strTemp1);
						// Done with "command" key.
						RegCloseKey(hKey4);
					}
					// Done with "edit" key.
					RegCloseKey(hKey3);
				}
				// Done with "Shell" key.
				RegCloseKey(hKey2);
			}
			// Done with "FlashQuiz.document" key.
			RegCloseKey(hKey1);
		}
	}
}

// Function sets the specified Registry key's default (or "unnamed")
// value to the specified string.
// Return value: 0 if successful.
LONG CFlashQuizApp::Set_Default_Value(HKEY hKey, LPCTSTR pszValue)
{
	DWORD dwBuffSize = strlen(pszValue) + 1;
	return RegSetValueEx(hKey, NULL, 0, REG_SZ, (const BYTE *) pszValue, dwBuffSize);
}

// Ensures that the specified file (which must exist!) is a Windows bitmap.
BOOL CFlashQuizApp::Check_Bitmap_Header(CString& strFileName)
{
	CFile file;
	BITMAPFILEHEADER bfh;
	BITMAPINFO bmi;
	BOOL bResult = TRUE;

	if (file.Open(strFileName, CFile::modeRead | CFile::shareDenyNone, NULL))
	{
		try
		{
			// Try to read a bitmap file header.
			if (file.Read(&bfh, sizeof(bfh)) != sizeof(bfh))
				bResult = FALSE;
			// bType field should contain "BM"
			if (bResult)
				if (bfh.bfType != 0x4D42)
					bResult = FALSE;
			// File size should match the size recorded in the header.
			if (bResult)
				if (file.GetLength() != bfh.bfSize)
					return FALSE;
			// File should then contain a BITMAPINFO structure.
			if (bResult)
				if (file.Read(&bmi, sizeof(bmi)) != sizeof(bmi))
					bResult = FALSE;
		}
		// Catch the exception that may be thrown if the read functions fail.
		catch (CFileException * e)
		{
			e->Delete();
			bResult = FALSE;
		}
	}
	else
		bResult = FALSE;
	file.Close();
	return bResult;
}


// Creates/modifies [windir]\WININIT.INI
// to remove our component files the next time the system starts.
// This is only used for Win 9x. For Win NT, the MoveFileEx API is used.
BOOL CFlashQuizApp::Do_File_Removal(void)
{
	int			pos;
	BOOL		bExists = FALSE,	// WININIT.INI exists -- default = NO.
				bNoRename = TRUE,	// [Rename] Section not found -- default = YES.
				bEndOfFile,
				bStatus = TRUE;		// Default return status = TRUE.
	CString		strTemp,			// Scratch string.
				strWinPath,			// Path to Windows "Home" directory.
				strWinInit,			// Filespec for WININIT.INI.
				strWinNew,			// Filespec for WININIT.LMO.
				strMyPath,			// Path to our "home" directory.
				strFullPath;		// Fully-qualified path to the current file.
	CStdioFile	fileIn,				// Input file.
				fileOut;			// Output file.

	GetEnvironmentVariable("windir", g_szBuffer, 1024);		// Get windows "Home" directory.
	strWinPath = g_szBuffer;								// Save it.
	if (strWinPath.Right(1) != "\\")						// Append backslash if not already present.
		strWinPath += '\\';
	strWinInit = strWinPath + "WININIT.INI";				// Build filespec for WININIT.INI
	strWinNew = strWinPath + "WININIT.FQ";					// Build filespec for WININIT.LMO (our working copy)
	strTemp = theApp.m_pszHelpFilePath;						// Get our path.
	pos = strTemp.ReverseFind('\\');						// Find last backslash.
	strMyPath = strTemp.Left(pos + 1);						// Truncate at last backslash.
	GetShortPathName(strMyPath, g_szBuffer, 2048);			// Convert to "short" name.
	strMyPath = g_szBuffer;									// Copy short name back to our path.
	// Create our output file.
	if (!fileOut.Open(strWinNew, CFile::modeCreate | CFile::modeWrite | CFile::shareExclusive | CFile::typeText))
		return FALSE;	// Signal failure if we can't create WININIT.FQ
	// See if WININIT.INI is already there.
	if (fileIn.Open(strWinInit, CFile::modeRead | CFile::shareExclusive | CFile::typeText))
	{
		bExists = TRUE;										// Set flag that we're working with an existing WININIT.INI
		do													// Loop through the file.
		{
			try
			{
				bEndOfFile = !fileIn.ReadString(strTemp);	// Read a line.
			}
			catch (CFileException *e)
			{
				e->Delete();								// Bail if error.
				bStatus = FALSE;
				break;
			}
			strTemp += '\n';								// Restore trailing CR/LF
			try
			{
				fileOut.WriteString(strTemp);				// Copy line to working file.
			}
			catch(CFileException *e)
			{
				e->Delete();								// Bail if write fails.
				bStatus = FALSE;
				break;
			}
			strTemp.TrimLeft();								// Trim leading/trailing spaces.
			strTemp.TrimRight();
			if (strTemp.CompareNoCase("[Rename]") == 0)		// Is it the [Rename] section tag?
			{
				bNoRename = FALSE;							// Yes .. clear flag.
				break;										// Stop copying.
			}
		}
		while (!bEndOfFile);								// Loop until end of WININIT.INI is reached.
	}
	if (bStatus)	// If no errors so far:
	{
		try
		{
			if (bNoRename)									// If [Rename] section tag was not found,
				fileOut.WriteString("\n[Rename]\n");		// write it to the file.
			strFullPath.Format("%s%s", strMyPath, "FLASHQUIZ.EXE");
			if (GetShortPathName(strFullPath, g_szBuffer, 2048) != 0)			// Convert to "short" name.
			{
				strTemp.Format("NUL=%s\n", g_szBuffer);	// Now add lines to the [Rename] section to
				fileOut.WriteString(strTemp);					// delete our component files.
			}
			strFullPath.Format("%s%s", strMyPath, "FLASHQUIZ.HLP");
			if (GetShortPathName(strFullPath, g_szBuffer, 2048) != 0)			// Convert to "short" name.
			{
				strTemp.Format("NUL=%s\n", g_szBuffer);	// Now add lines to the [Rename] section to
				fileOut.WriteString(strTemp);					// delete our component files.
			}
			strFullPath.Format("%s%s", strMyPath, "FLASHQUIZ.CNT");
			if (GetShortPathName(strFullPath, g_szBuffer, 2048) != 0)			// Convert to "short" name.
			{
				strTemp.Format("NUL=%s\n", g_szBuffer);	// Now add lines to the [Rename] section to
				fileOut.WriteString(strTemp);					// delete our component files.
			}
			strFullPath.Format("%s%s", strMyPath, "FLASHQUIZ.FTS");
			if (GetShortPathName(strFullPath, g_szBuffer, 2048) != 0)			// Convert to "short" name.
			{
				strTemp.Format("NUL=%s\n", g_szBuffer);	// Now add lines to the [Rename] section to
				fileOut.WriteString(strTemp);					// delete our component files.
			}
			strFullPath.Format("%s%s", strMyPath, "FLASHQUIZ.GID");
			if (GetShortPathName(strFullPath, g_szBuffer, 2048) != 0)			// Convert to "short" name.
			{
				strTemp.Format("NUL=%s\n", g_szBuffer);	// Now add lines to the [Rename] section to
				fileOut.WriteString(strTemp);					// delete our component files.
			}
		}
		catch (CFileException *e)
		{
			e->Delete();									// Bail if any of the above writes fail.
			bStatus = FALSE;
		}
	}
	if (bStatus)						// If no errors so far,
	{
		if (bExists && !bEndOfFile)								// If WININIT.INI exists and we haven't copy all of it,
		{
			do													// Copy the rest of the WININIT.INI to WININIT.LMO
			{
				try
				{
					bEndOfFile = !fileIn.ReadString(strTemp);
				}
				catch (CFileException *e)
				{
					e->Delete();
					bStatus = FALSE;
					break;
				}
				strTemp += '\n';
				try
				{
					fileOut.WriteString(strTemp);
				}
				catch(CFileException *e)
				{
					e->Delete();
					bStatus = FALSE;
					break;
				}
			}
			while (!bEndOfFile);
		}
	}
	if (bExists)			// If WININIT.INI exists,
		fileIn.Close();		// Close it.
	fileOut.Close();		// Close WININIT.LMO.
	if (bStatus)								// If no errors so far,
	{
		if (bExists)							// Does WININIT.INI exist?
		{
			if (unlink(strWinInit) != 0)		// Yes .. delete it. If delete fails,
			{
				unlink(strWinNew);				// delete WININIT.LMO
				return FALSE;					// and signal failure to caller.
			}
		}
		rename(strWinNew, strWinInit);			// Rename WININIT.LMO to WININIT.INI
	}
	else					// If there were any errors,
	{
		unlink(strWinNew);	// Delete WININIT.LMO.
	}
	return bStatus;			// Return final status to caller.
}


// Checks the file association. Returns TRUE if
// HKCR\.fqz does not exist, or it does exist and
// its (default) value contains our document type.
// Otherwise, returns FALSE.
BOOL CFlashQuizApp::Check_Association()
{
	LONG	lResult;
	int		iResult;
	DWORD	dwType, dwBuffSize;
	HKEY	hKey1;
	char	szBuff1[_MAX_PATH];
	CString strTemp1, strTemp2;
	BOOL	bFound = FALSE;

	// Get fully-qualified path to our .EXE file.
	strTemp1 = m_pszHelpFilePath;
	iResult = strTemp1.ReverseFind('.');
	strTemp2 = strTemp1.Left(iResult);
	strTemp2.MakeLower();
	strTemp2 += ".exe";
	// Is there a key for the .FQZ extension?
	lResult = RegOpenKeyEx(HKEY_CLASSES_ROOT, ".fqz", 0, KEY_ALL_ACCESS, &hKey1);
	if (lResult == 0)	// If so, retrieve its default value.
	{
		dwBuffSize = _MAX_PATH;
		lResult = RegQueryValueEx(hKey1, NULL, 0, &dwType, (LPBYTE) szBuff1, &dwBuffSize);
		if (lResult == 0)
		{
			if (dwType == REG_SZ)
			{
				// If default string contains our document type name.
				if (stricmp(szBuff1, szDocType) == 0)
				{
					bFound = TRUE;	// Set flag.
				}
			}
		}
		RegCloseKey(hKey1);
	}
	else
		bFound = TRUE;
	return bFound;
}

void CFlashQuizApp::Do_Uninstall()
{
	int				pos;
	BOOL			bIsNT;
	CString			strTemp, strMyPath;
	OSVERSIONINFO	vi;

	vi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&vi);
	bIsNT = vi.dwPlatformId == VER_PLATFORM_WIN32_NT;

	strTemp = theApp.m_pszHelpFilePath;						// Get our path.
	pos = strTemp.ReverseFind('\\');						// Find last backslash.
	strMyPath = strTemp.Left(pos + 1);						// Truncate at last backslash.

	// Delete Registry keys. Note that since this program runs under NT
	// as well as Win 9x, we must delete keys one at a time. Under NT,
	// RegDeleteKey() will not remove a key that has subkeys.
	if (Check_Association())
	{
		strTemp.Format("%s\\DefaultIcon", szDocType);
		RegDeleteKey(HKEY_CLASSES_ROOT, strTemp);
		strTemp.Format("%s\\Shell\\Open\\Command", szDocType);
		RegDeleteKey(HKEY_CLASSES_ROOT, strTemp);
		strTemp.Format("%s\\Shell\\Open", szDocType);
		RegDeleteKey(HKEY_CLASSES_ROOT, strTemp);
		strTemp.Format("%s\\Shell\\Edit\\Command", szDocType);
		RegDeleteKey(HKEY_CLASSES_ROOT, strTemp);
		strTemp.Format("%s\\Shell\\Edit", szDocType);
		RegDeleteKey(HKEY_CLASSES_ROOT, strTemp);
		strTemp.Format("%s\\Shell", szDocType);
		RegDeleteKey(HKEY_CLASSES_ROOT, strTemp);
		RegDeleteKey(HKEY_CLASSES_ROOT, szDocType);
		RegDeleteKey(HKEY_CLASSES_ROOT, ".fqz");
	}
	if (bIsNT)	// Under NT, we can use MoveFileEx()
	{
		strTemp.Format("%sFlashQuiz.exe", strMyPath);
		MoveFileEx(strTemp, NULL, MOVEFILE_DELAY_UNTIL_REBOOT);
		strTemp.Format("%sFlashQuiz.hlp", strMyPath);
		MoveFileEx(strTemp, NULL, MOVEFILE_DELAY_UNTIL_REBOOT);
		strTemp.Format("%sFlashQuiz.cnt", strMyPath);
		MoveFileEx(strTemp, NULL, MOVEFILE_DELAY_UNTIL_REBOOT);
		strTemp.Format("%sFlashQuiz.fts", strMyPath);
		MoveFileEx(strTemp, NULL, MOVEFILE_DELAY_UNTIL_REBOOT);
		strTemp.Format("%sFlashQuiz.gid", strMyPath);
		MoveFileEx(strTemp, NULL, MOVEFILE_DELAY_UNTIL_REBOOT);
	}
	else	// Under Win 9x, we must use WININIT.INI to do the work.
	{
		Do_File_Removal();
	}
	RegDeleteKey(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\FlashQuiz");
	::MessageBox(NULL, "FlashQuiz's data has been removed from the Registry.\n"
		"Component files will be removed the next time you start the system.", "FlashQuiz Uninstall", MB_ICONINFORMATION);
}

BOOL CFlashQuizApp::Find_Image_File(LPCTSTR szFileName, CString &strFileName)
{
	int		iPos;
	CFile	file;
	CString strTemp;

	if (file.Open(szFileName, CFile::modeRead | CFile::shareDenyNone, NULL))
	{
		file.Close();
		strFileName = szFileName;
		return TRUE;
	}
	else
	{
		iPos = strlen(szFileName) - 1;
		while (iPos >= 0)
		{
			if (szFileName[iPos] == '\\')
				break;
			--iPos;
		}
		if (iPos >= 0)
		{
			strTemp = &szFileName[iPos + 1];
			if (file.Open(strTemp, CFile::modeRead | CFile::shareDenyNone, NULL))
			{
				file.Close();
				strFileName = strTemp;
				return TRUE;
			}
		}
	}
	return FALSE;
}
