// MultChoice.cpp : implementation file
//
// FlashQuiz v1.0
// Copyright  2000 Ziff Davis Media, Inc.
// Written by Gregory A. Wolking.

#include "stdafx.h"
#include "flashquiz.h"
#include "MultChoice.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CFlashQuizApp theApp;
extern CFlashQuizDoc theDoc;

/////////////////////////////////////////////////////////////////////////////
// CMultChoice dialog


CMultChoice::CMultChoice(CWnd* pParent /*=NULL*/)
	: CDialog(CMultChoice::IDD, pParent)
{
	int i;
	//{{AFX_DATA_INIT(CMultChoice)
	//}}AFX_DATA_INIT

	for (i = 0; i < 5; i++)
		m_bAnswer[i] = FALSE;
	m_pQuestion = NULL;
	m_bPreviewMode = FALSE;
	m_intReduceTimer = -1;
	m_bDisableTimer = FALSE;
	m_bIsMaximized = FALSE;
	m_bHasCustomFont = FALSE;
	m_hBitmap = NULL;
}


void CMultChoice::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMultChoice)
	DDX_Control(pDX, IDC_fraChoose, m_fraChoose);
	DDX_Control(pDX, IDC_fraStatus, m_fraStatus);
	DDX_Control(pDX, IDOK, m_cmdOK);
	DDX_Control(pDX, IDC_cmdQuit, m_cmdQuit);
	DDX_Control(pDX, IDC_lblScore, m_lblScore);
	DDX_Control(pDX, IDC_lblQuizTime, m_lblQuizTime);
	DDX_Control(pDX, IDC_lblQuestionTime, m_lblQuestionTime);
	DDX_Control(pDX, IDC_lblCumulative, m_lblCumulative);
	DDX_Control(pDX, IDC_fraGraphic, m_fraGraphic);
	DDX_Control(pDX, IDC_txtQuestion, m_txtQuestion);
	//}}AFX_DATA_MAP
	// Class wizard doesn't recognize control arrays, so we have to
	// put this outside the Class Wizards' reserved box.
	DDX_Control(pDX, IDC_chkAnswer0, m_chkAnswer[0]);
	DDX_Control(pDX, IDC_chkAnswer1, m_chkAnswer[1]);
	DDX_Control(pDX, IDC_chkAnswer2, m_chkAnswer[2]);
	DDX_Control(pDX, IDC_chkAnswer3, m_chkAnswer[3]);
	DDX_Control(pDX, IDC_chkAnswer4, m_chkAnswer[4]);
	DDX_Check(pDX, IDC_chkAnswer0, m_bAnswer[0]);
	DDX_Check(pDX, IDC_chkAnswer1, m_bAnswer[1]);
	DDX_Check(pDX, IDC_chkAnswer2, m_bAnswer[2]);
	DDX_Check(pDX, IDC_chkAnswer3, m_bAnswer[3]);
	DDX_Check(pDX, IDC_chkAnswer4, m_bAnswer[4]);
}


BEGIN_MESSAGE_MAP(CMultChoice, CDialog)
	//{{AFX_MSG_MAP(CMultChoice)
	ON_BN_CLICKED(IDC_cmdQuit, OncmdQuit)
	ON_WM_CLOSE()
	ON_WM_SIZE()
	ON_EN_KILLFOCUS(IDC_txtQuestion, OnKillfocustxtQuestion)
	ON_EN_SETFOCUS(IDC_txtQuestion, OnSetfocustxtQuestion)
	//}}AFX_MSG_MAP
	ON_CONTROL_RANGE(BN_CLICKED, IDC_chkAnswer0, IDC_chkAnswer4, OnchkAnswer)
	ON_MESSAGE(MY_UPDATE_TIMER, OnUpdateTimer)
	ON_MESSAGE(MY_QUESTION_TIMEOUT, OnQuestionTimeout)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMultChoice message handlers

BOOL CMultChoice::OnInitDialog() 
{
	int			i;
	CString		strTemp,
				strImageFile;

	// Call base class first to create the window and controls.
	CDialog::OnInitDialog();
	m_intTextStart = m_intTextEnd = 0;
	strTemp.Format("Question %u", m_intQuestion);
	SetWindowText(strTemp);
	m_intAnswerCount = 0;
	m_intReduceTimer = -1;
	m_bReviewMode = m_bDisableTimer = m_bHasCustomFont = FALSE;
	if (m_bPreviewMode)
	{
		m_cmdOK.ShowWindow(SW_HIDE);
		m_cmdQuit.SetWindowText("End Preview");
	}
	if (m_pQuestion->m_bCustomFont)		// Does this question have a custom font?
	{
		// Yes. Try to create the font.
		if (m_fontCustom.CreateFontIndirect(m_pQuestion->m_pFont))
		{
			m_txtQuestion.SetFont(&m_fontCustom);	// Apply the font to the question field.
			m_bHasCustomFont = TRUE;	// Set flag so we know to release the font in OnDestroy().
		}
	}
	for (i = 0; i < 5; i++)
	{
		// Multiple answers allowed?
		if (m_pQuestion->m_bAllowMultiple)
		{
			// Yes -- make the answer buttons checkboxes and include all of them in the tab order.
			m_chkAnswer[i].ModifyStyle(BS_AUTORADIOBUTTON, BS_AUTOCHECKBOX | WS_TABSTOP);
		}
		else
		{
			// No -- make them radio buttons, but only include the first button in the tab order.
			if (i == 0)
				m_chkAnswer[i].ModifyStyle(BS_AUTOCHECKBOX, BS_AUTORADIOBUTTON);
			else
				m_chkAnswer[i].ModifyStyle(BS_AUTOCHECKBOX | WS_TABSTOP, BS_AUTORADIOBUTTON);
		}
		if (!m_pQuestion->m_strAnswers[i].IsEmpty())	// Does this answer exist?
		{
			++m_intAnswerCount;	// Yes ... increment the answer count.
		}
		m_iAnswerIndex[i] = -1;	// Initialize the index entry.
	}
	if (theDoc.m_bRandomA)	// Is "randomize answers" enabled?
	{
		// Yes -- randomize the index.
		if (!theApp.Randomize_List(m_iAnswerIndex, m_intAnswerCount))
		{
			MessageBox("Memory allocation error", "FlashQuiz", MB_ICONINFORMATION);
			EndDialog(IDCANCEL);
			return TRUE;
		}
	}
	else
	{
		// No -- set the index to natural order.
		for (i = 0; i < m_intAnswerCount; i++)
			m_iAnswerIndex[i] = i;
	}
	m_intActualScore = m_pQuestion->m_intScore;		// Set initial score.
	Update_Point_Value();							// Update score display.
	if (theDoc.m_intPossibleScore == 0 || m_bPreviewMode)	// Display cumulative score.
	{
		m_lblCumulative.SetWindowText("Score so far: none.");
	}
	else
	{
		strTemp.Format("Score so far: %u of %u points (%u%%)",
			theDoc.m_intTotalScore,
			theDoc.m_intPossibleScore,
			(100 * theDoc.m_intTotalScore) / theDoc.m_intPossibleScore);
		m_lblCumulative.SetWindowText(strTemp);
	}
	if (theDoc.m_bTimeLimit)						// Does question have a time limit?
	{
		theApp.Update_Quiz_Time(m_lblQuizTime);		// Yes ... update the quiz time display.
	}
	else
	{
		m_lblQuizTime.ShowWindow(SW_HIDE);			// No ... hide the quiz time display.
	}
	m_intTimeLeft = m_pQuestion->m_intTime;			// Initialize time left.
	if (m_intTimeLeft == 0)							// Is there a time limit for this question?
	{
		m_lblQuestionTime.ShowWindow(SW_HIDE);		// No .. hide question time display.
	}
	else
	{
		Update_Question_Time();						// Yes .. update question time display,
		if (m_pQuestion->m_intTimeDeduct > 0)		// and if "reduce score by" option is set,
		{
			m_intReduceTimer = 0;					// initialize the reduction interval.
		}
	}
	m_txtQuestion.SetWindowText(m_pQuestion->m_strQuestion);	// Display question text.
	for (i = 0; i < 5; i++)							// For each possible answer:
	{
		if (m_iAnswerIndex[i] < 0)					// Does this answer exist?
		{
			m_chkAnswer[i].ShowWindow(SW_HIDE);		// No .. hide the checkbox/radio button.
		}
		else										// Yes .. format the answer text.
		{
			strTemp.Format("&%c. %s", 'A' + i, m_pQuestion->m_strAnswers[m_iAnswerIndex[i]]);
			m_chkAnswer[i].SetWindowText(strTemp);
			m_chkAnswer[i].EnableWindow(TRUE);
		}
	}
	for (i = 0; i < m_intAnswerCount; i++)			// Initialize answer array.
		m_bAnswer[m_iAnswerIndex[i]] = FALSE;
	m_bShowText = !m_pQuestion->m_strQuestion.IsEmpty();		// Set flag if question has text.
	m_bShowGraphic = !m_pQuestion->m_strBitmapFile.IsEmpty();	// Set flag if question has a graphic.
	if (m_bShowGraphic)		// If question has a graphic,
	{
		// Does the graphic file exist?
		if (theApp.Find_Image_File(m_pQuestion->m_strBitmapFile, strImageFile))
			// Yes ... try to load the bitmap.
			m_hBitmap = (HBITMAP) LoadImage(NULL, strImageFile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
		else
		{
			// No ... display error message.
			m_bDisableTimer = TRUE;
			strTemp.Format("Error:\nCould not find or open image file %s", "FlashQuiz", strImageFile);
			MessageBox(strTemp, "FlashQuiz", MB_ICONINFORMATION);
			m_bDisableTimer = FALSE;
			m_hBitmap = NULL;
		}
	}
	// Show/hide and position the question text/image controls.
	Set_Question_Position();
	// Note: When answer controls are radio buttons, there must be
	// at least one button checked, otherwise the buttons will not
	// be included in the tab order. As a result, the user won't
	// be able to tab to the buttons until he either clicks on one
	// of them or uses its keyboard accelerator to check that button.
	if (!m_pQuestion->m_bAllowMultiple)
		m_bAnswer[0] = TRUE;
	UpdateData(FALSE);
	if (m_bIsMaximized)		// If we were maximized when destroyed,
	{
		ShowWindow(SW_SHOWMAXIMIZED);	// Maximize again.
	}
	m_pQuestion->m_bPresented = TRUE;	// Mark question as presented to the user.
	m_chkAnswer[0].SetFocus();
	return FALSE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

// Exit via OnClose()
void CMultChoice::OncmdQuit() 
{
	OnClose();
}

// Override this to kill the {Esc} key.
void CMultChoice::OnCancel()
{
	OnClose();
}

void CMultChoice::OnClose() 
{
	// If the "time expired" message is active, ignore the event.
	// Dialog will be closed automatically after the timer expires.
	if (!m_bPreviewMode)
	{
		m_bDisableTimer = TRUE;
		if (MessageBox("Are you sure you want to abort this quiz?", "FlashQuiz", MB_YESNO | MB_ICONQUESTION) == IDYES)
			EndDialog(IDCANCEL);
		else
			m_bDisableTimer = FALSE;
	}
	else
		EndDialog(IDCANCEL);
}

// Handler for answer checkboxes/radio buttons.
// Prevents user from changing his selection while in "review" mode.
void CMultChoice::OnchkAnswer(int nID) 
{
	UpdateData(!m_bReviewMode);
}

void CMultChoice::OnOK() 
{
	int		i;
	BOOL	bStatus;

	if (!m_bPreviewMode)
	{
		if (m_bReviewMode)
		{
			if (theDoc.m_bTimeLimit && theDoc.m_intTimeLeft < 0)
				EndDialog(IDCANCEL);
			else
				EndDialog(IDOK);
			return;
		}
		UpdateData(TRUE);
		bStatus = FALSE;
		for (i = 0; i < m_intAnswerCount; i++)
			bStatus = bStatus || m_bAnswer[i];
		if (!bStatus)
		{
			MessageBox("You must choose at least one answer!", "FlashQuiz", MB_ICONINFORMATION);
			return;
		}
		bStatus = TRUE;
		for (i = 0; i < m_intAnswerCount; i++)
		{
			m_pQuestion->m_bAnswers[m_iAnswerIndex[i]] = m_bAnswer[i];
			bStatus = bStatus && (m_bAnswer[i] == m_pQuestion->m_bCorrect[m_iAnswerIndex[i]]);
		}
		if (bStatus)
		{
			theDoc.m_intTotalScore += m_intActualScore;
			m_pQuestion->m_intActualScore = m_intActualScore;
		}
		else
		{
			if (theDoc.m_bShowCorrect)
				Review_Answer(FALSE);
			else
				EndDialog(IDOK);
			return;
		}
	}
	CDialog::OnOK();
}

// Rearranges controls when window size changes.
void CMultChoice::OnSize(UINT nType, int cx, int cy) 
{
	int		i;
	// Default control sizes, margin, and vertical gaps between controls, all in DLUs.
	CRect	rectMargins(0, 0, 7, 7),
			rectButton(0, 0, 138, 14),
			rectStatus(0, 0, 323, 29),
			rectChoose(0, 0, 323, 80),
			rectGap1(0, 0, 5, 5),
			rectGap2(0, 0, 9, 9),
			rectGap3(0, 0, 13, 13),
			rectLabel1(0, 0, 150, 8),
			rectLabel2(0, 0, 137, 8),
			rectCheckBox(0, 0, 200, 10);

	// Can't move windows that don't exist yet!
	if (m_cmdOK.m_hWnd == NULL)
		return;
	// Set or clear "maximized" flag accordingly.
	m_bIsMaximized = (nType == SIZE_MAXIMIZED);
	// Convert all DLUs to pixels.
	MapDialogRect(rectMargins);
	MapDialogRect(rectButton);
	MapDialogRect(rectStatus);
	MapDialogRect(rectChoose);
	MapDialogRect(rectGap1);		// Gap between "Status" frame and "Submit/Abort" buttons.
	MapDialogRect(rectGap2);	// Gap between "Status" frame and the labels within.
	MapDialogRect(rectGap3);	// Gap between the "Choose" frame and the checkboxes within.
	MapDialogRect(rectLabel1);
	MapDialogRect(rectLabel2);
	MapDialogRect(rectCheckBox);
	// Move the buttons to the bottom corners of the window.
	rectButton.OffsetRect(rectMargins.right, cy - rectButton.bottom - rectMargins.bottom);
	m_cmdOK.MoveWindow(rectButton);
	rectButton.OffsetRect(cx - rectButton.right - rectMargins.right, 0);
	m_cmdQuit.MoveWindow(rectButton);
	// "Status" frame goes above the buttons, separated by the first gap.
	rectStatus.left = rectMargins.right;
	rectStatus.right = cx - rectMargins.right;
	rectStatus.OffsetRect(0, rectButton.top - rectStatus.bottom - rectGap1.bottom);
	m_fraStatus.MoveWindow(rectStatus);
	// Position the labels within the "Status" frame.
	rectLabel1.OffsetRect(rectStatus.left - rectLabel1.left + rectMargins.right, rectStatus.top - rectLabel1.top + rectGap2.bottom);
	m_lblScore.MoveWindow(rectLabel1);
	rectLabel2.OffsetRect(rectStatus.right - rectLabel2.right - rectMargins.right, rectLabel1.top - rectLabel2.top);
	m_lblQuestionTime.MoveWindow(rectLabel2);
	rectLabel1.OffsetRect(0, rectGap2.bottom);
	m_lblCumulative.MoveWindow(rectLabel1);
	rectLabel2.OffsetRect(0, rectGap2.bottom);
	m_lblQuizTime.MoveWindow(rectLabel2);
	// "Choose" frame goes just above the "Status" frame, with no gap between them.
	rectChoose.left = rectStatus.left;
	rectChoose.right = rectStatus.right;
	rectChoose.OffsetRect(0, rectStatus.top - rectChoose.bottom);
	m_fraChoose.MoveWindow(rectChoose);
	// Position the first checkbox relative to the "Choose" frame.
	rectCheckBox.left = rectChoose.left + rectMargins.right;		// Checkbox fills the frame horizontally, with
	rectCheckBox.right = rectChoose.right - rectMargins.right;		// appropriate margins on either side.
	rectCheckBox.OffsetRect(0, rectChoose.top - rectCheckBox.top + rectGap3.bottom);	// Top edge is relative to top edge of the frame.
	m_chkAnswer[0].MoveWindow(rectCheckBox);
	// Position each remaining checkbox relative to the one above it.
	for (i = 1; i < 5; i++)
	{
		rectCheckBox.OffsetRect(0, rectGap3.bottom);
		m_chkAnswer[i].MoveWindow(rectCheckBox);
	}
	CDialog::OnSize(nType, cx, cy);
	Set_Question_Position();
}

/////////////////////////////////////////////
// App-specific message handlers.

// Handles message posted by the main dialog's "master timer" handler.
LONG CMultChoice::OnUpdateTimer(WPARAM, LPARAM)
{
	CString strTemp;

	if (m_bDisableTimer || m_bReviewMode)
	{
		if (theDoc.m_bTimeLimit)
			++theDoc.m_intTimeLeft;
		return 0L;
	}
	if (m_intReduceTimer >= 0)
	{
		++m_intReduceTimer;
		if (m_intReduceTimer >= m_pQuestion->m_intTimeDeduct)
		{
			m_intReduceTimer = 0;
			m_intActualScore -= m_pQuestion->m_intScoreDeduct;
			if (m_intActualScore < 1)
				m_intActualScore = 1;
			Update_Point_Value();
		}
	}
	if (theDoc.m_intTimeLeft < 0)
	{
		m_bDisableTimer = TRUE;
		PostMessage(MY_QUESTION_TIMEOUT, 0, 1);
	}
	else
		theApp.Update_Quiz_Time(m_lblQuizTime);
	if (m_intTimeLeft > 0)
	{
		--m_intTimeLeft;
		Update_Question_Time();
		if (m_intTimeLeft == 0)
		{
			m_bDisableTimer = TRUE;
			PostMessage(MY_QUESTION_TIMEOUT, 0, 0);
		}
	}
	return 0L;
}

// Handles message posted when time limit expires.
// lParam == 0 -> Question time limit expired.
// lParam == 1 -> Quiz time limit expired.
LONG CMultChoice::OnQuestionTimeout(WPARAM, LPARAM lParam)
{
	theApp.ShowQuestionTimeout(this, lParam);
	if (theDoc.m_bShowCorrect)
		Review_Answer(TRUE);
	else
	{
		if (lParam == 0)
			EndDialog(IDOK);
		else
			EndDialog(IDCANCEL);
	}
	return 0L;
}

///////////////////////////////////////////
// Helper functions.

// Indicates correct answer when user answers wrong or
// quiz/question time limit expires.
void CMultChoice::Review_Answer(BOOL bTimeout)
{
	int		i;
	CDC		*pDC;
	CRect	rect;
	CBrush	*pBrush;

	m_bReviewMode = TRUE;	// Set flag for other event handlers.
	if (!bTimeout)
		MessageBox("Your answer was incorrect.\nClick OK to see the correct answer.", "FlashQuiz", MB_ICONINFORMATION);
	pDC = GetDC();			// Get client window device context.
	// Select the "hollow" brush.
	pBrush = (CBrush*) pDC->SelectStockObject(HOLLOW_BRUSH);
	for (i = 0; i < m_intAnswerCount; i++)
	{
		if (m_pQuestion->m_bCorrect[m_iAnswerIndex[i]])	// Is answer correct?
		{
			m_chkAnswer[i].GetWindowRect(rect);	// Get window rect for corresponding control
			ScreenToClient(rect);				// Convert to client coordinates.
			rect.InflateRect(1, 1, 1, 1);		// Inflate by one pixel.
			pDC->Rectangle(rect);				// Draw a rectangle around the control.
		}
	}
	pDC->SelectObject(pBrush);	// Restore original brush.
	ReleaseDC(pDC);				// Release the device context.
	// Change the "Submit" button's text.
	m_cmdOK.SetWindowText("&Next question");
	m_cmdOK.SetFocus();
	// Show appropriate status message.
	if (bTimeout)
		m_lblScore.SetWindowText("Time expired!");
	else
		m_lblScore.SetWindowText("Incorrect!");
	m_lblCumulative.SetWindowText("Correct answer shown above.");
	return;
}

// Updates the point value display.
void CMultChoice::Update_Point_Value()
{
	CString strTemp;

	if (m_intActualScore == 1)
	{
		m_lblScore.SetWindowText("This question is worth 1 point.");
	}
	else
	{
		strTemp.Format("This question is worth %u points.", m_intActualScore);
		m_lblScore.SetWindowText(strTemp);
	}
}

// Positions the question text and/or graphic controls.
// Called whenever the window size changes.
void CMultChoice::Set_Question_Position()
{
	CRect rect1, rect2, rectMargins(0, 0, 7, 7);

	GetClientRect(&rect1);
	m_fraChoose.GetWindowRect(&rect2);
	ScreenToClient(&rect2);
	if (!m_bShowGraphic)	// No bitmap?
	{
		m_fraGraphic.ShowWindow(SW_HIDE);		// Hide the graphic.
		rect2.bottom = rect2.top - 2;
		rect2.top = rect1.top + rectMargins.bottom;
		rect2.right = (rect2.Width() / 2) + rect2.left;
		rect2.OffsetRect(((rect1.Width() - rect2.Width()) / 2), 0);
		m_txtQuestion.MoveWindow(&rect2);
	}
	else
	{
		if (!m_bShowText)	// No question text?
		{
			m_txtQuestion.ShowWindow(SW_HIDE);		// Hide the text.
			rect2.bottom = rect2.top - 2;
			rect2.top = rect1.top + rectMargins.bottom;
			rect2.right = (rect2.Width() / 2) + rect2.left;
			rect2.OffsetRect((rect1.Width() - rect2.Width()) / 2, 0);
			m_fraGraphic.MoveWindow(&rect2);
		}
		else									// Got both bitmap and text
		{
			rect2.bottom = rect2.top - 2;
			rect2.top = rect1.top + rectMargins.bottom;
			rect2.right = (rect2.Width() / 2) + rect2.left - (rectMargins.right / 2);
			m_fraGraphic.MoveWindow(&rect2);	// Move windows to proper position.
			rect2.OffsetRect(rect1.right - rect2.right - rectMargins.right, 0);
			m_txtQuestion.MoveWindow(&rect2);
		}
		// Reload & rescale the bitmap.
		m_fraGraphic.Set_Scaled_Bitmap(m_hBitmap, FALSE);
		// Control will resize itself to fit the bitmap, so we need to
		// re-center it in the desired area.
		m_fraGraphic.GetWindowRect(rect1);
		ScreenToClient(rect1);
		rect1.OffsetRect((rect2.Width() - rect1.Width()) / 2, (rect2.Height() - rect1.Height()) / 2);
		m_fraGraphic.MoveWindow(&rect1);
	}
}

// Updates the question time display.
void CMultChoice::Update_Question_Time()
{
	CString strTemp;

	strTemp.Format("Time left for this question: %u seconds", m_intTimeLeft);
	m_lblQuestionTime.SetWindowText(strTemp);

}


BOOL CMultChoice::DestroyWindow() 
{
	if (m_bHasCustomFont)					// If custom font was assigned,
	{
		m_txtQuestion.SetFont(GetFont());	// Revert to default font.
		m_fontCustom.DeleteObject();		// Delete the font object.
		m_bHasCustomFont = FALSE;			// Clear the flag.
	}
	if (m_hBitmap != NULL)			// If image is loaded,
	{
		DeleteObject(m_hBitmap);	// delete it
		m_hBitmap = NULL;			// and NULL its handle.
	}
	return CDialog::DestroyWindow();
}

void CMultChoice::OnKillfocustxtQuestion() 
{
	m_txtQuestion.GetSel(m_intTextStart, m_intTextEnd);	
}

void CMultChoice::OnSetfocustxtQuestion() 
{
	m_txtQuestion.SetSel(m_intTextStart, m_intTextEnd);
}
